class MapHandler {
  map: L.Map;
  overlays: OverlayState;
  layers: TileLayerWrapper[];
  modals: ModalCollection;
  
  static instance: MapHandler | null = null;
  
  private constructor(map: L.Map, overlays: OverlayState, layers: TileLayerWrapper[], modals: ModalCollection) {
    this.map = map;
    this.overlays = overlays;
    this.layers = layers;
    this.modals = modals;
  }
  
  static init(map: L.Map, overlays: OverlayState, layers: TileLayerWrapper[], modals: ModalCollection): void {
    if (!MapHandler.instance) {
      MapHandler.instance = new MapHandler(map, overlays, layers, modals);
    }
  }

  static setButtonClick(btnId: string, handler: any): void {
      const button = document.getElementById(btnId);
      if (button) {
        button.onclick = handler;
      }
  }
  
  static resetMapClick(): void {
    const self = MapHandler.instance;
    if (self) {
      try {
        const addPointBtn = document.getElementById("addPoint-btn");
        if (addPointBtn) {
          addPointBtn.classList.remove("activeBtn");
        }
        self.map.off("click", MapHandler.addMarker);
      } catch {}
      try {
        const addCircleBtn = document.getElementById("addCircle-btn");
        if (addCircleBtn) {
          addCircleBtn.classList.remove("activeBtn");
        }
        self.map.off("click", MapHandler.addCircle);
      } catch {}
      try {
        const addPolygonBtn = document.getElementById("addPolygon-btn");
        if (addPolygonBtn) {
          addPolygonBtn.classList.remove("activeBtn");
        }
        self.map.off("click", MapHandler.polygonAddPoint);
      } catch {}
    }
  }
  
  static addMarker(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.modals.cancel.setVisible(false);
      self.modals.createOverlay.setState(OverlayType.POINT, {
        latlng: e.latlng,
        map: self.map,
        overlays: self.overlays,
      });    
      MapHandler.resetMapClick();
      self.modals.createOverlay.setVisible(true);
    }
  }
  
  static addCircle(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.modals.cancel.setVisible(false);
      self.modals.createOverlay.setState(OverlayType.CIRCLE, {
        latlng: e.latlng,
        map: self.map,
        overlays: self.overlays,
      });    
      MapHandler.resetMapClick();
      self.modals.createOverlay.setVisible(true);
    }
  }
  
  static addPolygon(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.modals.okCancel.setVisible(false);
      self.modals.createOverlay.setState(OverlayType.POLYGON, {
        points: self.overlays.polyline.points,
        map: self.map,
        overlays: self.overlays,
      });
      MapHandler.resetMapClick();
      self.overlays.polyline.clearPoints();
      self.modals.createOverlay.setVisible(true);
    }
  }
  
  static circleCollect(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.modals.closeAll();
      MapHandler.resetMapClick();
      (e.target as HTMLElement).classList.add("activeBtn");
      self.map.on("click", MapHandler.addCircle);
      
      const cancelBtn = self.modals.cancel.cancelBtn();
      if (cancelBtn) {
        cancelBtn.onclick = ()=> {
          self.modals.closeAll();
          MapHandler.resetMapClick();
        };
      }
      self.modals.cancel.setMsg("Placing circle");
      self.modals.cancel.setVisible(true);
    }
  }
  
  static markerCollect(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.modals.closeAll();
      MapHandler.resetMapClick();
      (e.target as HTMLElement).classList.add("activeBtn");
      self.map.on("click", MapHandler.addMarker);
      
      const cancelBtn = self.modals.cancel.cancelBtn();
      if (cancelBtn) {
        cancelBtn.onclick = ()=> {
          self.modals.closeAll();
          MapHandler.resetMapClick();
        };
      }
      self.modals.cancel.setMsg("Placing marker");
      self.modals.cancel.setVisible(true);
    }
  }
  
  static polygonCollect(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.modals.closeAll();
      MapHandler.resetMapClick();
      (e.target as HTMLElement).classList.add("activeBtn");
      
      // show cancel -- on cancel, clear polyline and reset map handling
      const cancelBtn = self.modals.cancel.cancelBtn();
      if (cancelBtn) {
        cancelBtn.onclick = MapHandler.polygonClearPoints;
      }
      self.modals.cancel.setMsg("Creating polygon");
      self.modals.cancel.setVisible(true);
      
      self.map.on("click", MapHandler.polygonAddPoint);
    }
  }
  
  static polygonClearPoints(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.modals.closeAll();
      self.overlays.polyline.clearPoints();
      MapHandler.resetMapClick();
    }
  }
  
  static polygonAddPoint(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.overlays.polyline.insertPoint(e.latlng);
      if (self.overlays.polyline.numPoints() >= 3) {
        self.modals.cancel.setVisible(false);
        
        const okBtn = self.modals.okCancel.okBtn();
        if (okBtn) {
          okBtn.onclick = MapHandler.addPolygon;
        }
        const cancelBtn = self.modals.okCancel.cancelBtn();
        if (cancelBtn) {
          cancelBtn.onclick = MapHandler.polygonClearPoints;
        }
        self.modals.okCancel.setMsg("Creating polygon");
        self.modals.okCancel.setVisible(true);
      }
    }
  }
  
  static overlaySave(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.modals.closeAll();
      MapHandler.resetMapClick();
      (e.target as HTMLElement).classList.add("activeBtn");
      self.modals.okCancel.setMsg("Save current map overlays?");

      const okBtn = self.modals.okCancel.okBtn();
      if (okBtn) {
        okBtn.onclick = ()=> {
          OverlayState.save(self.overlays);
          self.modals.okCancel.setVisible(false);
          // show info modal "Save complete" 
        }
      }
      const cancelBtn = self.modals.okCancel.cancelBtn();
      if (cancelBtn) {
        cancelBtn.onclick = () => {
          self.modals.okCancel.setVisible(false);
        }
      }
      self.modals.okCancel.setVisible(true);
    }
  }
  
  static overlayClear(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      self.modals.closeAll();
      MapHandler.resetMapClick();
      (e.target as HTMLElement).classList.add("activeBtn");
      self.modals.okCancel.setMsg("Clear all map overlays (will not affect saved data)?");

      const okBtn = self.modals.okCancel.okBtn();
      if (okBtn) {
        okBtn.onclick = ()=> {
          self.overlays = OverlayState.clear(self.overlays, self.map);
          self.modals.okCancel.setVisible(false);
        }
      }
      const cancelBtn = self.modals.okCancel.cancelBtn();
      if (cancelBtn) {
        cancelBtn.onclick = () => {
          self.modals.okCancel.setVisible(false);
        }
      }
      self.modals.okCancel.setVisible(true);
    }
  }
  
  static swapTiles(e: any): void {
    const self = MapHandler.instance;
    if (self) {
      if (TileLayerWrapper.getActiveLayer() == "satelliteLayer") {
        TileLayerWrapper.enableOnly("streetLayer", self.map);
      } else {
        TileLayerWrapper.enableOnly("satelliteLayer", self.map);
      }
    }
  }
}